﻿namespace InfragisticsGridExample
{
    using System;
    using System.Data;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;
    using System.Windows.Media;

    using Infragistics.Windows.DataPresenter;
    using Infragistics.Windows.DataPresenter.Events;
    using Infragistics.Windows.Editors;

    /// <summary>
    ///     Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            this.InitializeComponent();
            this.viewModel = new ViewModel();
            this.DataContext = this.viewModel;
        }

        public readonly ViewModel viewModel;

        private void ChangeDataTable_Click(object sender, RoutedEventArgs e)
        {
            // Change DataTable columns
            this.viewModel.ChangeDataTable();
        }

        private void Lock_Click(object sender, RoutedEventArgs e)
        {
            bool isLockedSetting = true;
            this.SetIsLocked(isLockedSetting);
        }

        private void Unlock_Click(object sender, RoutedEventArgs e)
        {
            bool isLockedSetting = false;
            this.SetIsLocked(isLockedSetting);
        }

        private void SetIsLocked(bool isLockedSetting)
        {
            // Iterate selected items Cells to set IsLocked
            foreach (var cell in this.dataGrid.SelectedItems.Cells)
            {
                var dataRowView = cell.Record.DataItem as DataRowView;
                if (dataRowView != null)
                {
                    var row = dataRowView.Row;
                    var parameter = row[cell.Field.Index] as Parameter;
                    if (parameter != null)
                    {
                        parameter.IsLocked = isLockedSetting;
                    }
                }
            }

            // Iterate SelectedItems Records to set IsLocked
            foreach (var record in this.dataGrid.SelectedItems.Records)
            {
                var dataRecord = record as DataRecord;
                if (dataRecord != null)
                {
                    var dataRowView = dataRecord.DataItem as DataRowView;
                    if (dataRowView != null)
                    {
                        var row = dataRowView.Row;
                        foreach (var item in row.ItemArray)
                        {
                            if (item.GetType() != typeof(DateTime))
                            {
                                var parameter = item as Parameter;
                                if (parameter != null)
                                {
                                    parameter.IsLocked = isLockedSetting;
                                }
                            }
                        }
                    }
                }
            }

            // Iterate SelectedItems Fields to set IsLocked
            foreach (var field in this.dataGrid.SelectedItems.Fields)
            {
                foreach (var record in this.dataGrid.Records)
                {
                    var dataRecord = record as DataRecord;
                    if (dataRecord != null)
                    {
                        var dataRowView = dataRecord.Cells[field.Index].Record.DataItem as DataRowView;
                        if (dataRowView != null)
                        {
                            var row = dataRowView.Row;
                            var parameter = row[field.Index] as Parameter;
                            if (parameter != null)
                            {
                                parameter.IsLocked = isLockedSetting;
                            }
                        }
                    }
                }
            }
        }

        private void dataGrid_ClipboardPasting(object sender, ClipboardPastingEventArgs e)
        {
            var values = e.Values;
            foreach (var item in values)
            {
                // Set any empty cells with a default value
                if ((string)item.Value == "")
                {
                    item.Value = "0.0000";
                }
            }
        }

        private void dataGrid_FieldLayoutInitialized(object sender, FieldLayoutInitializedEventArgs e)
        {
            var lockToColourConverter = new LockToColourConverter();
            var fieldWidth = new FieldLength(75);

            foreach (DataColumn column in this.viewModel.Data.Columns)
            {
                if (column.ColumnName != "Date")
                {
                    var localParameterField = new UnboundField
                                                    {
                                                        Label = column.ColumnName,
                                                        BindingPath = new PropertyPath(string.Format("[{0}].Value", column.ColumnName)),
                                                        BindingMode = BindingMode.TwoWay,
                                                        Settings =
                                                            {
                                                                EditorStyle = BuildStyleForDoubleTextEditor(),
                                                                CellValuePresenterStyle = BuildStyleForLockableCell(column.ColumnName, lockToColourConverter),
                                                            },
                                                        Width = fieldWidth
                                                    };
                    
                    e.FieldLayout.Fields.Add(localParameterField);
                }
                else
                {
                    var dateField = new UnboundField
                                        {
                                            Label = column.ColumnName, 
                                            BindingPath = new PropertyPath(string.Format("[{0}]", column.ColumnName)), 
                                            Settings =
                                                {
                                                    EditorStyle = BuildStyleForDateTextEditor(),
                                                    CellValuePresenterStyle = BuildStyleForDateCellValuePresenter()
                                                },
                                            Width = fieldWidth
                                        };

                    e.FieldLayout.Fields.Add(dateField);
                }
            }
        }

        private static Style BuildStyleForDateCellValuePresenter()
        {
            // cell styling for date field
            var dateCellValuePresenterStyle = new Style(typeof(CellValuePresenter));
            var borderBrushSetter = new Setter(BorderBrushProperty, Brushes.LightGray);
            var borderThicknessSetter = new Setter(BorderThicknessProperty, new Thickness(0, 0, 1, 1));
            dateCellValuePresenterStyle.Setters.Add(borderBrushSetter);
            dateCellValuePresenterStyle.Setters.Add(borderThicknessSetter);
            return dateCellValuePresenterStyle;
        }

        private static Style BuildStyleForDateTextEditor()
        {
            // text editor format styling for date field
            var dateTextEditorStyle = new Style(typeof(XamTextEditor));
            var dateFormatSetter = new Setter(ValueEditor.FormatProperty, "dd/MM/yyyy");
            var readOnlySetter = new Setter(ValueEditor.IsReadOnlyProperty, true);
            dateTextEditorStyle.Setters.Add(dateFormatSetter);
            dateTextEditorStyle.Setters.Add(readOnlySetter);
            return dateTextEditorStyle;
        }

        private static Style BuildStyleForDoubleTextEditor()
        {
            // text editor format styling for fields displaying doubles
            var doubleTextEditorStyle = new Style(typeof(XamTextEditor));
            doubleTextEditorStyle.Setters.Add(new Setter(ValueEditor.FormatProperty, "f4"));
            return doubleTextEditorStyle;
        }

        private static Style BuildStyleForLockableCell(string columnKey, LockToColourConverter lockToColourConverter)
        {
            // lock cell styling for Parameter object fields
            var backgroundColourBinding = new Binding { Path = new PropertyPath(string.Format("DataItem.Row[{0}].IsLocked", columnKey)), Converter = lockToColourConverter, };
            var lockCellValuePresenterStyle = new Style(typeof(CellValuePresenter));
            lockCellValuePresenterStyle.Setters.Add(new Setter(BackgroundProperty, backgroundColourBinding));
            lockCellValuePresenterStyle.Setters.Add(new Setter(BorderBrushProperty, Brushes.LightGray));
            lockCellValuePresenterStyle.Setters.Add(new Setter(BorderThicknessProperty, new Thickness(0, 0, 1, 1)));
            return lockCellValuePresenterStyle;
        }

        private void dataGrid_SelectedItemsChanged(object sender, SelectedItemsChangedEventArgs e)
        {
            // A workaround to the problem of the Parameter fields' XamTextEditor not applying its style's "f4" format
            // to edited cells. This problem occurs in VolAnalyser, but, for some mysterious reason, not in this test app!
            // We need to set the XamTextEditor style's Format property to "f4" again at the point when items are selected
            // in order for their edited cells to be displayed in the f4 format.

            // Iterate selected items Cells to set text editor formatting
            foreach (var cell in this.dataGrid.SelectedItems.Cells)
            {
                if (cell.Field.Label.ToString() != "Date")
                {
                    cell.Field.Settings.EditorStyle = BuildStyleForDoubleTextEditor();
                }
            }

            // Iterate selected items Records to set text editor formatting
            foreach (var record in this.dataGrid.SelectedItems.Records)
            {
                foreach (var field in record.FieldLayout.Fields)
                {
                    if (field.Label.ToString() != "Date")
                    {
                        field.Settings.EditorStyle = BuildStyleForDoubleTextEditor();
                    }
                }
            }

            // Iterate selected items Fields to set text editor formatting
            foreach (var field in this.dataGrid.SelectedItems.Fields)
            {
                if (field.Label.ToString() != "Date")
                {
                    field.Settings.EditorStyle = BuildStyleForDoubleTextEditor();
                }
            }
        }
    }
}