﻿namespace InfragisticsGridExample
{
    using System;
    using System.ComponentModel;
    using System.Windows;
    using System.Windows.Media;

    public class Parameter : ParameterBase, INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public Parameter(int parameterNumber, double value, bool isLocked)
            : base(parameterNumber)
        {
            this.IsLocked = isLocked;
            this.Initialize(value);
        }

        /// <summary>
        ///     Initializes a new instance of the <see cref="ParameterLocal" /> class.
        /// </summary>
        /// <param name="parameterNumber">The parameter number.</param>
        /// <param name="value">The value.</param>
        public Parameter(int parameterNumber, double value)
            : base(parameterNumber)
        {
            this.Initialize(value);
        }

        public Parameter(Parameter that)
            : base(that.Ordinal)
        {
            this.initialValue = that.initialValue;
            this.localValue = that.localValue;
            this.autoLock = that.autoLock;
            this.isLocked = that.isLocked;
        }

        public bool AutoLock
        {
            get
            {
                return this.autoLock;
            }

            set
            {
                if (this.autoLock != value)
                {
                    this.autoLock = value;
                    this.NotifyPropertyChanged("AutoLock");
                }
            }
        }

        public bool CanReset
        {
            get
            {
                return this.initialValue != this.localValue;
            }
        }

        public string ErrorMessage
        {
            get
            {
                return this.errorMessage;
            }

            set
            {
                if (this.errorMessage != value)
                {
                    this.errorMessage = value;
                    this.NotifyPropertyChanged("ErrorMessage");
                }
            }
        }

        public bool HasError
        {
            get
            {
                return this.hasError;
            }

            set
            {
                if (this.hasError != value)
                {
                    this.hasError = value;
                    this.NotifyPropertyChanged("HasError");
                }
            }
        }

        public bool IsLocked
        {
            get
            {
                return this.isLocked;
            }

            set
            {
                if (this.isLocked != value)
                {
                    this.isLocked = value;
                    this.NotifyPropertyChanged("IsLocked");
                }
            }
        }

        public Brush LockBrush
        {
            get
            {
                return this.lockBrush;
            }

            set
            {
                if (this.lockBrush != value)
                {
                    this.lockBrush = value;
                    this.NotifyPropertyChanged("LockBrush");
                }
            }
        }

        public double Value
        {
            get
            {
                return this.localValue;
            }

            set
            {
                try
                {
                    if (this.localValue != value)
                    {
                        if (double.IsNaN(this.initialValue))
                        {
                            this.initialValue = value;
                        }
                        this.localValue = value;
                        this.NotifyPropertyChanged("Value");
                        this.NotifyPropertyChanged("CanReset");

                        if (this.CanReset && this.AutoLock)
                        {
                            this.IsLocked = true;
                        }
                    }
                    this.HasError = false;
                }
                catch (Exception ex)
                {
                    this.ErrorMessage = ex.Message;
                    this.HasError = true;
                }
            }
        }

        public void Initialize(double value)
        {
            if (this.lockBrush == null)
            {
                // LinearGradientBrush lgb = new LinearGradientBrush(Colors.Transparent, Colors.DodgerBlue, new Point(0, 0.5), new Point(1, 0.5));
                var lgb = new LinearGradientBrush(Colors.Transparent, Colors.Yellow, new Point(0.5, 0), new Point(0.5, 1));

                //LinearGradientBrush lgb = new LinearGradientBrush();
                //lgb.StartPoint = new Point(0.5, 0);
                //lgb.EndPoint = new Point(0.5, 1);
                //lgb.GradientStops.Add(new GradientStop(Colors.DarkGray, 0.0));
                //lgb.GradientStops.Add(new GradientStop(Colors.DodgerBlue, 0.15));
                //lgb.GradientStops.Add(new GradientStop(Colors.Transparent, 0.5));
                //lgb.GradientStops.Add( new GradientStop(Colors.DodgerBlue, 0.85));
                //lgb.GradientStops.Add(new GradientStop(Colors.DarkGray, 1.0));

                this.LockBrush = Brushes.Yellow;
            }

            this.initialValue = value;
            this.localValue = value;
            this.NotifyPropertyChanged(null);
        }

        public void Reset()
        {
            this.Value = this.initialValue;
            this.IsLocked = false;
        }

        public override string ToString()
        {
            return this.Value.ToString();
        }

        private bool autoLock = true;

        private string errorMessage;

        private bool hasError;

        private double initialValue;

        private bool isLocked;

        private double localValue;

        private Brush lockBrush;

        private double value;

        private void NotifyPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}